#include <filesystem>
#include <iostream>

#include <fggl/gfx/window.hpp>
#include <fggl/gfx/ogl.hpp>
#include <fggl/gfx/renderer.hpp>
#include <fggl/gfx/shader.hpp>
#include <fggl/data/procedural.hpp>
#include <fggl/ecs/ecs.hpp>
#include <fggl/debug/debug.h>
#include <fggl/data/storage.hpp>

// prototype of resource discovery
void discover(std::filesystem::path base) {

	std::vector< std::filesystem::path > contentPacks;
	for ( auto& item : std::filesystem::directory_iterator(base) ) {

		// content pack detection
		if ( std::filesystem::is_directory( item ) ) {
			auto manifest = item.path() / "manifest.yml";
			if ( std::filesystem::exists( manifest ) ) {
				contentPacks.push_back( item.path() );
			}
		}
		
	}

	// what did we find?
	std::cerr << "found pack(s): " << std::endl;
	for ( auto& pack : contentPacks ) {
		std::cerr << pack << std::endl;
	}

}

int main(int argc, char* argv[]) {
	fggl::gfx::Context ctx;

	// build our main window
	fggl::gfx::Window win;
	win.fullscreen( true );

	// opengl time
	fggl::gfx::Graphics ogl(win);
	fggl::gfx::MeshRenderer meshRenderer;

	// debug layer
	fggl::debug::DebugUI debug(win);
	debug.visible(false);

	// storage API
	fggl::data::Storage storage;
	discover( storage.resolvePath(fggl::data::Data, "res") );

	fggl::gfx::ShaderCache cache(storage);

	fggl::gfx::ShaderConfig config;
	config.name = "unlit";
	config.vertex = "unlit_vert.glsl";
	config.fragment = "unlit_frag.glsl";
	auto shader = cache.load(config);

	// create ECS
	fggl::ecs::ECS ecs;
	ecs.registerComponent<fggl::gfx::MeshToken>();

	// create an entity
	auto entity = ecs.createEntity();

	// in a supprise to no one it's a triangle
	auto mesh = fggl::data::make_triangle();
	auto token = meshRenderer.upload(mesh);
	token.pipeline = shader;
	ecs.addComponent<fggl::gfx::MeshToken>(entity, token);

	while( !win.closeRequested() ) {
		ctx.pollEvents();
		debug.frameStart();

		// update step

		// render step
		ogl.clear();
		meshRenderer.render(win, ecs);

		debug.draw();
		win.swap();
	}

	return 0;
}
