
#include "procedural.hpp"
#include "model.hpp"

using namespace fggl::data;

fggl::data::Mesh fggl::data::make_triangle() {
    constexpr fggl::math::vec3 pos[]{
            {-0.5f, -0.5f, 0.0f},
            {0.5f,  -0.5f, 0.0f},
            {0.0f,  0.5f,  0.0f}
    };

    // add points
    fggl::data::Mesh mesh;
    for (auto po : pos) {
        Vertex vert{};
        vert.posititon = po;
        vert.colour = fggl::math::vec3(1.0f, 1.0f, 1.0f);
        mesh.push(vert);
    }

    // mesh
    for (int i = 0; i < 3; ++i) {
        mesh.pushIndex(i);
    }

    return mesh;
}

/*
constexpr float cubeVertex[8 * 3] = {
	0, 0, 0,
	0, 1, 0,
	1, 1, 0,
	1, 0, 0,
	0, 0, 1,
	0, 1, 1,
	1, 1, 1,
	0, 0, 0,
};

constexpr int cubeIndexes[6 * 4] = {
	7, 6, 5, 4,
	0, 1, 2, 3,
	6, 7, 3, 2,
	5, 6, 2, 1,
	4, 5, 1, 0,
	7, 4, 0, 3,
};


void quadsToTris(fggl::data::Mesh& data, const std::vector<unsigned int>& idx, const int* quads, const int nQuads) {
	for (int p = 0; p < nQuads; p++) {
		int offset = p * 4;
		// first tri
		data.pushIndex( idx[ quads[offset + 0 ] ] );
		data.pushIndex( idx[ quads[offset + 1 ] ] );
		data.pushIndex( idx[ quads[offset + 2 ] ] );

		// second tri
		data.pushIndex( idx[ quads[offset + 2 ] ] );
		data.pushIndex( idx[ quads[offset + 3 ] ] );
		data.pushIndex( idx[ quads[offset + 0 ] ] );
	}
}

inline glm::vec3 calcNormal(Vertex prev, Vertex next, Vertex curr) {
	return glm::cross( prev.posititon - curr.posititon, next.posititon - curr.posititon );
}

void computeNormals(Mesh& mesh) {

	const int idxes = mesh.indexCount();
	const int vertCount = mesh.vertexCount();

	// zero the normals
	for (int i=0; i<vertCount; i++) {
		Vertex& vert = mesh.vertex( i );
		vert.normal = glm::vec3(0.0f);
	}

	// calculate normals for all triangles
	for (int tri=0; tri<idxes; tri += 3) {
		// get next 3 points
		Vertex& pa = mesh.vertex( tri );
		Vertex& pb = mesh.vertex( tri + 1 );
		Vertex& pc = mesh.vertex( tri + 2 );

		pa.normal += calcNormal( pb, pc, pa );
		pb.normal += calcNormal( pc, pa, pb );
		pc.normal += calcNormal( pa, pb, pc );
	}

	// normalise
	for (int i=0; i<vertCount; i++) {
		Vertex& vert = mesh.vertex( i );
		vert.normal = glm::normalize( vert.normal );
	}

}

Mesh fggl::data::make_cube() {

	int nVerts = sizeof(cubeVertex) / sizeof(cubeVertex[0]) / 3;
	int nQuads = sizeof(cubeIndexes) / sizeof(cubeIndexes[0]) / 4;

	std::vector< unsigned int > vertIndex;

	Mesh cube;

	// stick the vertex data into the mesh
	for ( int i=0; i < nVerts; i++ ) {
		fggl::data::Vertex vert;
		vert.posititon = fggl::math::vec3( cubeVertex[i], cubeVertex[i + 1], cubeVertex[i+2] );

		auto idx = cube.pushVertex(vert);
		vertIndex.push_back(idx);
	}

	// triagulate the quads
	quadsToTris(cube, vertIndex, cubeIndexes, nQuads);
	computeNormals(cube);

	return cube;
}*/

